@@ -15,7 +15,7 @@
 
 # import re
 from types import SimpleNamespace
-from typing import Union as _Union
+from typing import Iterable, Union as _Union
 # from typing import Iterable as _Iterable
 from typing import TypeVar as _TypeVar
 from colorama import Fore as _Fore
@@ -25,6 +25,8 @@ import colemen_utilities.string_utils as _csu
 import collections
 from collections.abc import MutableMapping
 
+import colemen_utilities.type_utils as _typ
+
 def set_defaults(default_vals, obj,**kwargs):
     '''
         Sets default values on the dict provided, if they do not already exist or
@@ -217,6 +219,31 @@ def get_kwarg(key_name:_Union[list,str], default_val=False, value_type=None, **k
                     return kwargs[v_name]
     return default_val
 
+def get_kwarg_remove(key_name:_Union[list,str], default_val=False, value_type=None, **kwargs)->tuple:
+
+    from colemen_utilities.random_utils.rand_generation import gen_variations
+
+
+    kwargs = keys_to_lower(kwargs)
+    if isinstance(key_name, list) is False:
+        key_name = [key_name]
+    result = default_val
+    for name in key_name:
+        # generate basic variations of the name
+        varis = gen_variations(name)
+        for v_name in varis:
+            if v_name in kwargs:
+                if value_type is not None:
+                    if isinstance(kwargs[v_name], value_type) is True:
+                        result = kwargs[v_name]
+                        del kwargs[v_name]
+                else:
+                    result = kwargs[v_name]
+                    del kwargs[v_name]
+    return (result,kwargs)
+
+
+
 def get_arg(args:dict,key_name:_Union[list,str],default_val=False, value_type=None)->any:
     '''
         Get a key's value from a dictionary.
@@ -396,7 +423,7 @@ def has_keys(data:dict,keys:list,**kwargs):
         return False
     return True
 
-def remove_keys(data:dict,keys:_Union[list,str],**kwargs)->dict:
+def remove_keys(data:dict,keys:_Union[list,str],reverse:bool=False,comp_values:bool=False)->dict:
     '''
         Remove matching keys from a dictionary or keep only the matching keys.
 
@@ -410,14 +437,13 @@ def remove_keys(data:dict,keys:_Union[list,str],**kwargs)->dict:
         `keys` {list|str}
             A key or list of keys that will be removed from the dictionary.
 
-        Keyword Arguments
-        -------------------------
         [`reverse`=False] {bool}
             If True, all keys except the ones provided will be removed.
 
         [`comp_values`=False] {bool}
             If True, remove keys based on their values
 
+
         Return {dict}
         ----------------------
         The dict with keys filtered.
@@ -431,8 +457,8 @@ def remove_keys(data:dict,keys:_Union[list,str],**kwargs)->dict:
         `method_name`: remove_keys
         * @xxx [06-04-2022 10:23:17]: documentation for remove_keys
     '''
-    reverse = get_kwarg(['reverse'], False, (bool), **kwargs)
-    comp_values = get_kwarg(['comp_values'], False, (bool), **kwargs)
+    # reverse = get_kwarg(['reverse'], False, (bool), **kwargs)
+    # comp_values = get_kwarg(['comp_values'], False, (bool), **kwargs)
     keys = _lu.force_list(keys)
 
 
@@ -703,7 +729,6 @@ def find_dict_key_diff(dict_one:dict,dict_two:dict,**kwargs)->list:
     else:
         return _lu.find_list_diff(list(dict_two.keys()),list(dict_one.keys()))
 
-
 def flatten(d, parent_key='', sep='.'):
     '''
         Flatten a dictionary with dot_notation.
@@ -832,3 +857,120 @@ def has_attr(class_instance,keys):
             return key
 
     return False
+
+def set_attr_from_dict(
+    class_instance,
+    data:dict,
+    set_privates:bool=False,
+    exclude:_Union[str,list]=None,
+    values:_Union[str,list]=False
+    )->Iterable[str]:
+    '''
+        Set a class's attributes from a dictionary.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `class_instance` {any}
+            The class instance to assign attributes to.
+
+        `data` {dict}
+            The dictionary of data to assign to the class.
+
+        [`set_privates`=False] {bool}
+            If True, this will search for attributes beginning with a single and double underscore.
+
+        [`exclude`=None] {str,list}
+            If any of these values are matched in an attribute name, that attribute will not be set.
+            Excluded attributes are not considered missing, so they wont show up in the return value.
+
+        [`values`=False] {None,str,list}
+            If provided, the attribute must have one of these values in order to be set.
+            For example, to set defaults you could provide "None" as this argument,
+            then only attributes with a None value will be set.
+
+        Return {list}
+        ----------------------
+        A list of attributes that were not set.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 02-24-2023 08:47:49
+        `version`: 1.0
+        `method_name`: set_attr_from_dict
+        * @xxx [02-24-2023 08:52:22]: documentation for set_attr_from_dict
+    '''
+    if isinstance(data,(dict)) is False:
+        raise TypeError(f"The data must be a dictionary, received: {type(data)}")
+    if values is False:
+        values = []
+    else:
+        values = _lu.force_list(values)
+    exclude = _lu.force_list(exclude,allow_nulls=False)
+    missing = []
+    for k,v in data.items():
+        skip = False
+        is_missing = True
+        opts = [k]
+        if set_privates is True:
+            opts.append(f"_{k}")
+            opts.append(f"__{k}")
+
+        for opt in opts:
+            if skip is True:
+                continue
+            if _csu.array_in_string(exclude,opt,False):
+                is_missing = False
+                skip = True
+                continue
+            if hasattr(class_instance,opt):
+                if len(values) > 0:
+                    if getattr(class_instance,opt) not in values:
+                        is_missing = False
+                        skip = True
+                        continue
+                try:
+                    # print(f"setting: {opt}:{v}")
+                    setattr(class_instance,opt,v)
+                except AttributeError as e:
+                    continue
+                is_missing = False
+                continue
+        if is_missing is True:
+            missing.append(k)
+    return missing
+
+
+
+def values_to_strings(value:dict,bools_to_ints=False)->dict:
+    '''
+        Convert all values in the dict to strings.
+        ----------
+
+        Arguments
+        -------------------------
+        `value` {type}
+            The dict to convert.
+
+        [`bools_to_ints`=False] {bool}
+            if True, bools will be a string integer "1" for True and "0" for False.
+
+
+        Return {dict}
+        ----------------------
+        The dict with values converted to the strings.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 03-09-2023 07:52:42
+        `version`: 1.0
+        `method_name`: values_to_strings
+        * @xxx [03-09-2023 07:56:21]: documentation for values_to_strings
+    '''
+    return _typ.to_string(value,convert_values=True,bools_to_ints=bools_to_ints)
+
+
+# session_id = Column(Integer,ForeignKey('sessions.session_id'),nullable=True,default=None)
\ No newline at end of file
