@@ -15,6 +15,8 @@
     `created`: 06-03-2022 10:22:15
     `memberOf`: type_utils
 '''
+from typing import Union
+import json
 import colemen_utilities.dict_utils as _obj
 
 
@@ -146,7 +148,58 @@ def python_type_name(value):
         return results[0]
     return results
 
-def bool_to_string(value, **kwargs):
+def to_string(value,convert_values=False,bools_to_ints=False):
+    '''
+        Convert the value to a string.
+        ----------
+
+        Arguments
+        -------------------------
+        `value` {type}
+            The value to convert
+
+        [`convert_values`=False] {bool}
+            If True and the value is a list/dict, it will convert all values to strings.
+            Otherwise, it will jsonify the entire value.
+
+        [`bools_to_ints`=False] {bool}
+            if True, bools will be a string integer "1" for True and "0" for False.
+
+        Return {str}
+        ----------------------
+        The value converted to a string.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 03-09-2023 07:49:07
+        `memberOf`: type_utils
+        `version`: 1.0
+        `method_name`: to_string
+        * @xxx [03-09-2023 07:51:21]: documentation for to_string
+    '''
+    if isinstance(value,(bool)):
+        return bool_to_string(value,number=bools_to_ints)
+    if isinstance(value,(int,float)):
+        return str(value)
+    if isinstance(value,(dict,list)):
+        if convert_values is True:
+            if isinstance(value,(dict)):
+                nv = {}
+                for k,v in value.items():
+                    nv[k] = to_string(v)
+                return nv
+            if isinstance(value,(list)):
+                nv = []
+                for v in value:
+                    nv.append(to_string(v))
+                return nv
+        else:
+            return json.dumps(value)
+
+    return value
+
+def bool_to_string(value:bool,number=False):
     '''
         Converts a boolean value to a string representation.
 
@@ -157,8 +210,6 @@ def bool_to_string(value, **kwargs):
         `value` {bool}
             The boolean to convert
 
-        Keyword Arguments
-        -------------------------
         [`number`=False] {bool}
             if True, the result will be a string integer "1" for True and "0" for False.
 
@@ -176,9 +227,9 @@ def bool_to_string(value, **kwargs):
         `version`: 1.0
         `method_name`: bool_to_string
     '''
-    
 
-    number = _obj.get_kwarg(["number"], False, (bool), **kwargs)
+
+    # number = _obj.get_kwarg(["number"], False, (bool), **kwargs)
     result = None
     if value is True:
         result = "true"
@@ -231,7 +282,7 @@ def bool_to_int(value,default=0):
     if value is False:
         return 0
 
-def to_bool(value,default=False):
+def to_bool(value,default=False,null_default=False):
     '''
         Convert a string to its boolean equivalent.
 
@@ -241,9 +292,13 @@ def to_bool(value,default=False):
         -------------------------
         `value` {str}
             the value to convert.
+
         [`default`=False] {any}
             The default value to return if a boolean cannot be determined.
 
+        [`null_default`=False] {any}
+            The default value to return if the value is null.
+
         Return {bool}
         ----------------------
         The boolean equivalent if successful, the default value otherwise.
@@ -257,7 +312,8 @@ def to_bool(value,default=False):
         `method_name`: to_bool
         * @xxx [06-01-2022 13:42:09]: documentation for to_bool
     '''
-
+    if value is None:
+        return null_default
 
     if isinstance(value,(bool)):
         return value
@@ -268,6 +324,7 @@ def to_bool(value,default=False):
         return True
     if str(value).lower() in False_syns:
         return False
+
     return default
 
 def is_scalar(value:any,exclude_bool=False)->bool:
@@ -340,7 +397,7 @@ def is_list_of_dicts(value:any)->bool:
     output = False
     if isinstance(value,(list)):
         result = True
-        
+
         for itm in value:
             if isinstance(itm,(dict)) is False:
                 result = False
@@ -348,7 +405,16 @@ def is_list_of_dicts(value:any)->bool:
             output = True
     return output
 
-
+def to_number(value:Union[str,int,float],default=None):
+    from colemen_utilities.validate_utils.general import is_float,is_integer
+    if isinstance(value,(int,float)):
+        return value
+    if isinstance(value,(str)):
+        if is_float(value):
+            return float(value)
+        if is_integer(value):
+            return int(value)
+    return default
 
 
 
