@@ -142,10 +142,10 @@ def is_float(value:Union[str,float],negatives=True):
     if isinstance(value,(str)):
 
         # @Mstep [] determine the appropriate regex to use.
-        reg = r'^[0-9\.]*$'
+        reg = r'^[0-9]*\.[0-9]*$'
 
         if negatives is True:
-            reg = r'^[0-9-\.]*$'
+            reg = r'^[0-9-]*\.[0-9]*$'
 
         # @Mstep [] strip leading and trailing spaces.
         value = _csu.strip(value,[" "])
@@ -254,14 +254,74 @@ def to_hash_id(value:str,prefix:str):
         value= f"{prefix}_{value}"
 
 def crud_type(value:str):
-    if isinstance(value,(str)) is False:
+    aliases = {
+        "create":["create","c","1","cr","cre","crea","creat"],
+        "read":["read","r","2","re","rea"],
+        "update":["update","u","3","up","upd","upda","updat"],
+        "delete":["delete","d","4","de","del","dele","delet"],
+    }
+    if isinstance(value,(str,int)) is False:
         return False
+    if isinstance(value,(int)):
+        value = str(value)
+
     value = _csu.strip(value,[" "])
+    value = value.lower()
+
+    for k,v in aliases.items():
+        if value in v:
+            return k
+    return False
     valids = ["create","read","update","delete"]
     if value.lower() not in valids:
         return False
     return True
 
+def request_method(value:str)->Union[str,bool]:
+    '''
+        Validate that the value is an HTTP request method.
+        
+        This will match partial names as well.
+        ----------
+
+        Arguments
+        -------------------------
+        `value` {str}
+            The value to test
+
+        Return {str,bool}
+        ----------------------
+        The request name if successful, False otherwise.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 03-15-2023 11:48:27
+        `memberOf`: general
+        `version`: 1.0
+        `method_name`: request_method
+        * @xxx [03-15-2023 11:49:30]: documentation for request_method
+    '''
+    if isinstance(value,(str)) is False:
+        return False
+    valids = {
+        "options":["options","o","op","opt","opti","optio","option"],
+        "connect":["connect","c","co","con","conn","conne","connec"],
+        "delete":["delete","d","de","del","dele","delet"],
+        "patch":["patch","pa","pat","patc"],
+        "trace":["trace","t","tr","tra","trac"],
+        "head":["head","h","he","hea"],
+        "post":["post","po","pos"],
+        "get":["get","g","ge","get"],
+        "put":["put","pu","put"],
+    }
+    # valids = ["connect","delete","get","head","options","patch","post","put","trace"]
+    for k,v in valids.items():
+        if value.lower() in v:
+            return k
+
+    return False
+
 def required_keys(required:list,data:dict)->Union[list,None]:
     '''
         Confirm that the data contains all of the required keys.
